from __future__ import annotations

import subprocess
import sys
from dataclasses import dataclass
from typing import Any

from pdm import Project, termui


def twine(*cmd: str, **kwargs: Any) -> subprocess.CompletedProcess[str]:
    return subprocess.run(
        [sys.executable, "-m", "twine", *cmd], capture_output=True, text=True, **kwargs
    )


@dataclass
class Publisher:
    project: Project

    @property
    def ui(self) -> termui.UI:
        return self.project.core.ui

    @property
    def files(self) -> list[str]:
        dist = self.project.pyproject_file.parent / "dist"
        return [str(f) for g in {"*.whl", "*.tar.gz"} for f in dist.glob(g)]

    def publish(
        self, repo: str, username: str, password: str, dry_run: bool = False
    ) -> None:
        repo_flag_suffix = "" if repo.lower() in {"pypi", "testpypi"} else "-url"
        if not dry_run:
            with self.ui.open_spinner(f"Uploading to {repo}") as spin:
                resp = twine(
                    "upload",
                    "--non-interactive",
                    f"--repository{repo_flag_suffix}",
                    repo,
                    "--username",
                    username,
                    *self.files,
                    env={"TWINE_PASSWORD": password},
                )
                if resp.returncode:
                    spin.fail("Publish failed")
                    self.ui.echo(termui.red(resp.stderr), err=True)
                    resp.check_returncode()
                else:
                    spin.succeed("Publish successful")

        verb = "Uploaded" if not dry_run else "Would upload"
        self.ui.echo(
            f"{verb} {termui.cyan(self.project.meta.name)}"
            f" ({termui.bold(self.project.meta.version)}) to {repo}"
        )



from pdm import Core

from .command import PublishCommand
from .config import CONFIG


def main(core: Core) -> None:
    core.register_command(PublishCommand)
    for k, v in CONFIG.items():
        core.add_config(k, v)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pdm_publish']

package_data = \
{'': ['*']}

install_requires = \
['pdm>=1,<2', 'twine>=3,<4']

entry_points = \
{'pdm': ['publish = pdm_publish.__init__:main']}

setup_kwargs = {
    'name': 'pdm-publish',
    'version': '0.2.0',
    'description': 'A PDM plugin to publish to PyPI',
    'long_description': None,
    'author': 'Branch Vincent',
    'author_email': 'branchevincent@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/branchvincent/pdm-publish',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from argparse import ArgumentParser, Namespace
from typing import Any

import click
from pdm import Project, termui
from pdm.cli.actions import do_build
from pdm.cli.commands.base import BaseCommand
from pdm.exceptions import PdmUsageError

from .core import Publisher


class PublishCommand(BaseCommand):  # type: ignore
    """Upload artifacts to a remote repository"""

    name = "publish"

    def add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            "-r",
            "--repository",
            help="The repository name or url to publish the package to"
            " [env var: PDM_PUBLISH_REPO]",
        )
        parser.add_argument(
            "-u",
            "--username",
            help="The username to access the repository"
            " [env var: PDM_PUBLISH_USERNAME]",
        )
        parser.add_argument(
            "-P",
            "--password",
            help="The password to access the repository"
            " [env var: PDM_PUBLISH_PASSWORD]",
        )
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="Perform all actions except upload the package",
        )
        parser.add_argument(
            "--no-build",
            action="store_false",
            dest="build",
            help="Don't build the package before publishing",
        )

    def coalesce(
        self, project: Project, options: Namespace, key: str, **kwargs: Any
    ) -> str:
        value = getattr(options, key) or project.config.get(f"publish.{key}")
        if not value:
            value = click.prompt(key.capitalize(), **kwargs)
        return str(value)

    def handle(self, project: Project, options: Namespace) -> None:
        # Validation
        username = self.coalesce(project, options, "username")
        password = self.coalesce(project, options, "password", hide_input=True)

        # Build
        if options.build:
            do_build(project)

        # Check for files
        publisher = Publisher(project)
        if not publisher.files:
            raise PdmUsageError(
                f"No files to publish! Retry without {termui.bold('--no-build')}."
            )

        # Publish
        publisher.publish(
            repo=options.repository or project.config["publish.repo"],
            username=username,
            password=password,
            dry_run=options.dry_run,
        )



from pdm.project.config import ConfigItem

CONFIG = {
    "publish.repo": ConfigItem(
        description="PyPI repo name (pypi/testpypi) or url",
        default="pypi",
        env_var="PDM_PUBLISH_REPO",
    ),
    "publish.username": ConfigItem(
        description="PyPI username", default="__token__", env_var="PDM_PUBLISH_USERNAME"
    ),
    "publish.password": ConfigItem(
        description="PyPI password", env_var="PDM_PUBLISH_PASSWORD"
    ),
}
